<?php
/**
 * Google, Yahoo, Live Searchを検索するクラス
 *
 */
class MashupSearch
{
	public $offset = 0;
	public $num = 10;

	const GOOGLE = "google";
	const YAHOO = "yahoo";
	const LIVE = "live";
	const BAIDU = "baidu";

	private $searcher = array();

	private $query = "";
	
	protected $results;
	protected $resultHtml;

	/**
	 * コンストラクタ
	 */
	function __construct($searchengine)
	{
		$cacheDir = '/tmp/cache';

		foreach($searchengine as $enginename) {
			switch($enginename) {
				case MashupSearch::GOOGLE:
					require_once 'WebSearch/GoogleSearch.php';
					array_push($this->searcher, new GoogleSearch($cacheDir));
					break;
				case MashupSearch::YAHOO:
					require_once 'WebSearch/YahooSearch.php';
					array_push($this->searcher, new YahooSearch($cacheDir));
					break;
				case MashupSearch::LIVE:
					require_once 'WebSearch/LiveSearch.php';
					array_push($this->searcher, new LiveSearch($cacheDir));
					break;
				case MashupSearch::BAIDU:
					require_once 'WebSearch/BaiduSearch.php';
					array_push($this->searcher, new BaiduSearch($cacheDir));
					break;
			}
		}
	}
	
	/**
	 * Google, Yahoo, Live Searchをまとめて検索
	 */
	public function search($query, $offset)
	{
		$this->query = $query;
		$this->offset = $offset * $this->num;

		$this->results = array();
		foreach ($this->searcher as $search) {
			array_push($this->results, $search->search($this->query, $this->offset, $this->num));
		}
	}
	
	/**
	 * 結果をHTMLで返します
	 */
	public function getResult()
	{
		return $this->results;
	}
	
	/**
	 * 検索結果からHTMLを作成します
	 */
	public function makeResultHTML(WebResultSet $result)
	{
		$html = "";
		$name = $result->getName();

		$num = $this->offset + 1;
		foreach($result->result as $res) {
			$html .= "\t". '<li style="border: 1px solid #ffffff;">' . "\n";
			$id = $result->getId() . $num;
			$html .= "\t<a name=\"" . $id . "\" id=\"" . $id . "\"></a>\n";
			$html .= "\t\t" . '<div class="result_title">';
			$html .= '<a href="' . $res->url . '">';
			$html .= $res->title . "</a></div>\n";
			
			$html .= "\t\t" . '<div class="result_summary">';
			$html .= $this->moji_cut($res->description, 120) . "</div>\n";
			
			$html .= "\t\t". '<div class="result_url">';
			$html .= $this->moji_cut($res->url, 50) . "</div>\n";
			
			$html .= "\t</li>\n";
			
			$num++;
		}
		
		return $html;
	}

	private function moji_cut($str, $maxlen){
	    //末尾の空白＆改行コード除去
	    $new_str=trim($str);
	    //エンティティ変換
	    $new_str = htmlspecialchars($new_str);
	    //指定文字数でカットし、省略マークを挿入
	    if(strlen($new_str) > $maxlen){
	        $new_str=mb_substr($new_str, 0, $maxlen);
	        $new_str.="...";
	    }
	    return $new_str;
	}
}
?>
