<?php
require_once 'WebSearch.php';
require_once 'WebResult.php';
require_once 'WebResultSet.php';

/**
 * Baidu検索クラス
 */
class BaiduSearch extends WebSearch
{
	/**
	 * Baiduの場合のキャッシュPrefix
	 */
	protected $cachePrefix = "baidu";

	/**
	 * Baidu
	 */
	protected $name = "Baidu";

	/**
	 * Baiduを検索する
	 */
	protected function searchCore($query, $offset, $count)
	{
		 // 実際にBaiduを検索してしまう
		$url ='http://www.baidu.jp/s?';
		$url.='wd='.urlencode($query);
		$url.='&rn=' . $count . '&ie=utf-8&tn=baidujp';
		if($offset) {
			$url .= '&pn='.$offset;
		}

		// DOMDocumentを作る
		$html = file_get_contents($url);

		/**
		 * Parse the HTML source.
		 */
		require_once('./HTMLParser.class.php');
		$parser = new HTMLParser;
		$parser->setRuleFile('./xhtml1-transitional_dtd.inc.php');
		$parser->setRoot('html', array('lang' => 'ja'));
		$parser->setGenericParent('body');

		$parser->parse($html);
		return $parser->dump();
	}
	
	/**
	 * 検索結果を整形して返す
	 */
	protected function getResult($data)
	{
		// DOMDocumentを生成
		$doc = new DOMdocument("1.0","UTF-8");

		// Baidu検索結果取得(クエリには第一引数を)。そのhtmlを読ませる
		$doc->loadHTML($data);

		// DOMXPathインスタンスを生成してDOMDocumentを関連づける
		$xpath = new DOMXPath($doc);

		// 検索結果生成
		$webResultSet = new WebResultSet();
		// XPath式を評価する
		// 総件数取得
		$entries = $xpath->query("//h1/span");
		$countvar = array();
		$rcd = mbereg('検索した結果、関連ウェブは(約|)((\d|,)+)件', $entries->item(0)->nodeValue, $countvar);
		$count = str_replace(',','', $countvar[2]);
		$webResultSet->count = (int)($count);
		
		// 検索結果
		$entries = $xpath->query("//table[@class='n']/tr/td");

		foreach($entries as $nodelist) {
			// Title
			$title = "";
			$titlepath = $xpath->query("font[@size='3']", $nodelist);
			if($titlepath->item(0)->childNodes) {
				$title = $titlepath->item(0)->nodeValue;
			}
			else {
			}
			
			// URL
			$url = "";
			$urlpath = $xpath->query("font[@size='3']/a/@href", $nodelist);
			if($urlpath->item(0)->childNodes) {
				$url = $urlpath->item(0)->nodeValue;
			}
			else {
			}

			$webResult = new WebResult($title, $url);
			
			// 概要
			$descpath = $xpath->query("font[@size='-1']", $nodelist);
			$description = "";
			$childNodes = $descpath->item(0)->childNodes;
			if($childNodes) {
				foreach($childNodes as $descNode) {
					if($descNode->nodeName == 'span' || $descNode->nodeName == 'a') {
					}
					else {
						$description .= $descNode->nodeValue;
					}
				}
			}
			
			$webResult->description = $description;

			// BaiduのキャッシュURL
			if($descpath->length) {
				$cachepath = $xpath->query("a[@class='m']/@href", $descpath->item(0));
				if($cachepath->length) {
					$cacheUrl = $cachepath->item(0)->nodeValue;
					$webResult->cacheUrl = $cacheUrl;
				}
			}

			// 検索結果を収納
			array_push($webResultSet->result, $webResult);
		}

		return $webResultSet;
	}
}
?>
