<?php
require_once 'WebSearch.php';
require_once 'WebResult.php';
require_once 'WebResultSet.php';

/**
 * Google検索クラス
 */
class GoogleSearch extends WebSearch
{
	/**
	 * Googleの場合のキャッシュPrefix
	 */
	protected $cachePrefix = "google";

	/**
	 * Google
	 */
	protected $name = "Google";
	
	/**
	 *
	 */
	private $countstart = 0;

	/**
	 * Googleを検索する
	 */
	protected function searchCore($query, $offset, $count)
	{
		$this->countstart = $offset * $count;
		
		 // 実際にGoogleを検索してしまう
		$url ='http://www.google.co.jp/search?';
		$url.='q='.urlencode($query);
		$url.='&num=' . $count . '&hl=ja&lr=lang_ja&ie=utf-8&oe=utf-8';
		if($offset) {
			$url .= '&start='.$offset;
		}

		// DOMDocumentを作る
		$html = file_get_contents($url);

		/**
		 * Parse the HTML source.
		 */
		require_once('./HTMLParser.class.php');
		$parser = new HTMLParser;
		$parser->setRuleFile('./xhtml1-transitional_dtd.inc.php');
		$parser->setRoot('html', array('lang' => 'ja'));
		$parser->setGenericParent('body');

		$parser->parse($html);
		return $parser->dump();
	}
	
	/**
	 * 検索結果を整形して返す
	 */
	protected function getResult($data)
	{
		// DOMDocumentを生成
		$doc = new DOMdocument("1.0","UTF-8");

		// Google検索結果取得(クエリには第一引数を)。そのhtmlを読ませる
		$doc->loadHTML($data);

		// DOMXPathインスタンスを生成してDOMDocumentを関連づける
		$xpath = new DOMXPath($doc);

		// 検索結果生成
		$webResultSet = new WebResultSet();
		// XPath式を評価する
		// 総件数取得
		$entries = $xpath->query("//table[@bgcolor='#e5ecf9']/tr/td/font[@size='-1']/b[3]");
		$count = str_replace(',','',$entries->item(0)->nodeValue);
		$webResultSet->count = (int)($count);
		
		if($count < $this->countstart) {
			return $webResultSet;
		}
		
		// 検索結果
		$entries = $xpath->query("//div[@class='g']");

		foreach($entries as $nodelist) {
			// Title
			$title = "";
			$titlepath = $xpath->query("h2[@class='r']/a[@class='l']", $nodelist);
			if($titlepath->item(0)->childNodes) {
				$title = $titlepath->item(0)->nodeValue;
			}
			else {
				$titlepath = $xpath->query("table/tr/td[@class='j']/table/tr/td/h2[@class='r']", $nodelist);
				$title = $titlepath->item(0)->nodeValue;
			}
			
			// URL
			$url = "";
			$urlpath = $xpath->query("h2[@class='r']/a[@class='l']/@href", $nodelist);
			if($urlpath->item(0)->childNodes) {
				$url = $urlpath->item(0)->nodeValue;
			}
			else {
				$urlpath = $xpath->query("table/tr/td[@class='j']/table/tr/td/h2[@class='r']/a[@class='l']/@href", $nodelist);
				$url = $urlpath->item(0)->nodeValue;
			}

			$webResult = new WebResult($title, $url);
			
			// 概要
			$descpath = $xpath->query("table//td[@class='j' or @class='j hc']/font[@size='-1']", $nodelist);
			$description = "";
			$childNodes = $descpath->item(0)->childNodes;
			if(!$childNodes) {
				$descpath = $xpath->query("table//td[@class='j' or @class='j hc']/table/tr/td[@valign='top']/font[@size='-1']", $nodelist);
				$childNodes = $descpath->item(0)->childNodes;
			}

			if($childNodes) {
				foreach($childNodes as $descNode) {
					if($descNode->nodeName == 'span') {
						break;
					}
					$description .= $descNode->nodeValue;
				}
			}
			
			$webResult->description = $description;

			// GoogleのキャッシュURL
			if($descpath->length) {
				$cachepath = $xpath->query("a[@class='fl']/@href", $descpath->item(0));
				if($cachepath->length) {
					$cacheUrl = $cachepath->item(0)->nodeValue;
					$webResult->cacheUrl = $cacheUrl;
				}
			}

			// 検索結果を収納
			array_push($webResultSet->result, $webResult);
		}

		return $webResultSet;
	}
}
?>
